# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import logging

from copy import copy
from eve.utils import config, ParsedRequest

from superdesk.utc import utcnow
from superdesk.services import BaseService
from superdesk.publish.formatters.ninjs_formatter import NINJSFormatter
from superdesk import get_resource_service

logger = logging.getLogger('superdesk')


class PublishService(BaseService):
    """A service for publishing to the content api.

    Serves mainly as a proxy to the data layer.
    """

    formatter = NINJSFormatter()
    subscriber = {'config': {}}

    def publish(self, item, subscribers=[]):
        """Publish an item to content api.

        This must be enabled via ``PUBLISH_TO_CONTENT_API`` setting.

        :param item: item to publish
        """

        if not self._filter_item(item):
            doc = self.formatter._transform_to_ninjs(item, self.subscriber)
            now = utcnow()
            doc.setdefault('firstcreated', now)
            doc.setdefault('versioncreated', now)
            doc['subscribers'] = [str(sub['_id']) for sub in subscribers]
            logger.info('publishing %s to %s' % (doc['guid'], subscribers))
            return self._create_doc(doc)
        else:
            return None

    def create(self, docs, **kwargs):
        ids = []
        for doc in docs:
            ids.extend(self._create_doc(doc, **kwargs))
        return ids

    def _create_doc(self, doc, **kwargs):
        """Create a new item or update existing."""
        item = copy(doc)
        item.setdefault('_id', item.get('guid'))
        _id = item[config.ID_FIELD] = item.pop('guid')
        original = self.find_one(req=None, _id=_id)
        if original:
            self.update(_id, item, original)
            return _id
        else:
            return super().create([item], **kwargs)[0]

    def _filter_item(self, item):
        """
        Filter the item out if it matches any API Block filter conditions
        :param item:
        :return: True of the item is blocked, False if it is OK to publish it on the API.
        """
        # Get the API blocking Filters
        req = ParsedRequest()
        filter_conditions = list(get_resource_service('content_filters').get(req=req, lookup={'api_block': True}))

        # No API blocking filters
        if not filter_conditions:
            return False

        filter_service = get_resource_service('content_filters')
        for fc in filter_conditions:
            if filter_service.does_match(fc, item):
                logger.info('API Filter block {} matched for item {}.'.format(fc, item.get(config.ID_FIELD)))
                return True

        return False




# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import bs4
import xml.etree.ElementTree as etree  # noqa
from xml.etree.ElementTree import ParseError  # noqa

inline_elements = set([
    'a',
    'b',
    'i',
    'em',
    'img',
    'sub',
    'sup',
    'abbr',
    'bold',
    'span',
    'cite',
    'code',
    'small',
    'label',
    'script',
    'strong',
    'object',
])

# from https://developer.mozilla.org/en-US/docs/Web/HTML/Block-level_elements
BLOCK_ELEMENTS = (
    "address",
    "article",
    "aside",
    "blockquote",
    "br",
    "canvas",
    "dd",
    "div",
    "dl",
    "fieldset",
    "figcaption",
    "figure",
    "footer",
    "form",
    "h1",
    "h2",
    "h3",
    "h4",
    "h5",
    "h6",
    "header",
    "hgroup",
    "hr",
    "li",
    "main",
    "nav",
    "noscript",
    "ol",
    "output",
    "p",
    "pre",
    "section",
    "table",
    "tfoot",
    "ul",
    "video")


def get_text_word_count(text):
    """Get word count for given plain text.

    :param text: text string
    """
    return len(text.split())


def get_text(html, content='xml', lf_on_block=False):
    """Get plain text version of HTML element

    if the HTML string can't be parsed, it will be returned unchanged
    :param html: html string to convert to plain text
    :param str content: 'xml' or 'html'
    :param bool lf_on_block: if True, add a line feed on block elements' tail
    """
    try:
        if content == 'html':
            # FIXME: this is a fragile way of handling HTML, it will be properly fixed
            #        when lxml patch will be used
            html = html.replace('<br>', '<br/>').replace('</br>', '').replace('<hr>', ' ')
        root = etree.fromstringlist('<doc>{0}</doc>'.format(html))
        if lf_on_block:
            for elem in root.iterfind('.//'):
                if elem.tag in BLOCK_ELEMENTS:
                    elem.tail = (elem.tail or '') + '\n'
        text = etree.tostring(root, encoding='unicode', method='text')
        return text
    except ParseError:
        return html


def get_word_count(html):
    """Get word count for given html.

    :param html: html string to count
    """
    soup = bs4.BeautifulSoup(html.replace('<br>', ' ').replace('<hr>', ' '), 'html.parser')

    # first run to filter out inlines
    for elem in soup.find_all():
        if elem.name in inline_elements:  # ignore inline elements
            elem.unwrap()

    # re-parse without inline, it will merge sibling text nodes
    soup = bs4.BeautifulSoup(str(soup), 'html.parser')
    text = ' '.join(soup.find_all(text=True))
    return get_text_word_count(text)


def get_char_count(html):
    """Get character count for given html.

    :param html: html string to count
    """
    return len(get_text(html))
